#include <tidy.h>
#include <tidybuffio.h>
#include "util.h"
#include "module.h"

int repairdoc(TidyDoc doc, TidyBuffer *docbuf) {
    int rc;
    rc = tidyParseBuffer(doc, docbuf);
    if (rc < 0) {
        error("tidyParseBuffer() returned code %d", rc);
        return rc;
    }
    rc = tidyCleanAndRepair(doc);
    if (rc < 0) {
        error("tidyCleanAndRepair() returned code %d", rc);
        return rc;
    }
    rc = tidyRunDiagnostics(doc);
    if (rc < 0)
        error("tidyRunDiagnostics() returned code %d", rc);
    return rc;
}

int mod_tidy_onpagecomplete(void *userdata, pagecompletedata_t *data) {
    if (searchextradata(EXTRA_TIDY, "tidyDoc", data->extradata.data, data->extradata.len) != NULL)
        return -1;

    TidyBuffer origbuf = {0}, errbuf = {0};
    tidyBufAttach(&origbuf, (byte*)data->page, data->npage);

    TidyDoc tdoc = tidyCreate();
    tidyOptSetBool(tdoc, TidyForceOutput, yes);
    tidyOptSetInt(tdoc, TidyWrapLen, 4096);
    tidySetErrorBuffer(tdoc, &errbuf);

    int rc;
    rc = repairdoc(tdoc, &origbuf);
    if (rc < 0)
        goto cleanup;

    debug("encountered errors in doc: %s", errbuf.bp);

    extradata_t entry = {
        .type = EXTRA_TIDY,
        .key = "tidyDoc",
        .val = (void*)tdoc,
    };
    dynarr_push(data->extradata, entry);
cleanup:
    tidyBufFree(&errbuf);
    // TODO: Safety is unknown
    // tdoc still exists while corresponding tidyBuffer is detached
    tidyBufDetach(&origbuf); 
    return rc >= 0 ? 0 : rc;
}

int mod_tidy_onpagedestroy(void *userdata, pagecompletedata_t *data) {
    TidyDoc doc = searchextradata(EXTRA_TIDY, "tidyDoc", data->extradata.data, data->extradata.len);
    if (doc != NULL)
        tidyRelease(doc);
    return 0;
}

int mod_tidy_init(crawlermodule_t *entry) {
    *entry = (crawlermodule_t) {
        .init = entry->init,
        .onpagecomplete = mod_tidy_onpagecomplete,
        .onpagedestroy = mod_tidy_onpagedestroy,
    };
    return 0;
}
